package cd.casic.ci.process.engine.worker.base;


import cd.casic.ci.process.constant.CommandConstant;
import cd.casic.ci.process.engine.constant.EngineRuntimeConstant;
import cd.casic.ci.process.engine.constant.PipelineBehaviorConstant;
import cd.casic.ci.process.engine.context.ConstantContext;
import cd.casic.ci.process.engine.enums.ContextStateEnum;
import cd.casic.ci.process.engine.manager.LoggerManager;
import cd.casic.ci.process.engine.manager.RunContextManager;
import cd.casic.ci.process.engine.runContext.BaseRunContext;

import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.enums.MachineSystemEnum;
import cd.casic.ci.process.process.dataObject.base.PipBaseElement;
import cd.casic.ci.process.process.dataObject.log.PipTaskLog;
import cd.casic.ci.process.process.dataObject.machine.MachineInfo;
import cd.casic.ci.process.process.dataObject.task.PipTask;
import cd.casic.ci.process.process.service.machine.MachineInfoService;

import cd.casic.ci.process.process.service.resource.ResourceManagerService;
import cd.casic.ci.process.ssh.SshClient;
import cd.casic.ci.process.ssh.SshClientFactory;
import cd.casic.ci.process.ssh.WinRMHelper;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import jakarta.annotation.Resource;

import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import javax.swing.text.StringContent;
import java.util.Arrays;
import java.util.List;
import java.util.Map;


@Data
@Slf4j
public abstract class BaseWorker implements Runnable{
    // 一些属性
    @Resource
    private RunContextManager contextManager;
    private String contextKey;
    @Resource
    private MachineInfoService machineInfoService;
    @Resource
    private LoggerManager loggerManager;
    @Resource
    private ResourceManagerService resourceManagerService;


    @Override
    public void run() {
        if (StringUtils.isEmpty(contextKey)) {
            log.error("请设置当前worker的contextKey");
            return;
        }
        doWorker(contextKey);
    }

    public void doWorker(String contextKey){
        BaseRunContext context = contextManager.getContext(contextKey);
        if (context instanceof TaskRunContext taskRunContext){
            try {
                Map<String, Object> globalVariables = context.getGlobalVariables();
                Object fromError = globalVariables.get(PipelineBehaviorConstant.PIPELINE_EXECUTE_FROM_ERROR);
                if (Boolean.TRUE.equals(fromError)&&ContextStateEnum.HAPPY_ENDING.getCode().equals(taskRunContext.getState().get())) {
                    append(taskRunContext,"跳过执行"+CommandConstant.ENTER);
                } else{
                    if (this instanceof PassableWorker passableWorker) {
                        taskRunContext.changeContextState(ContextStateEnum.SUSPEND);
                        passableWorker.waitForPermission();
                    } else {
                        taskRunContext.changeContextState(ContextStateEnum.READY);
                        taskRunContext.changeContextState(ContextStateEnum.RUNNING);
                    }
                    execute(taskRunContext);
                }
            } catch (Exception e) {
                log.error("================worker执行报错：",e);
                // todo 根据配置决定失败是跳过还是直接失败，如果直接跳过，状态改为跳过，如果直接失败状态就改为失败
                PipBaseElement contextDef = taskRunContext.getContextDef();
                if (contextDef instanceof PipTask task) {
                    Map<String, Object> taskProperties = task.getTaskProperties();
                    Object taskSkip = taskProperties.get(PipelineBehaviorConstant.TASK_SKIP_KEY);
                    if (Integer.valueOf("1").equals(taskSkip)) {
                        taskRunContext.changeContextState(ContextStateEnum.SKIP_TO);
                        append(context,e.getMessage());
                    } else{
                        taskRunContext.changeContextState(ContextStateEnum.BAD_ENDING);
                        append(context,e.getMessage());
                    }
                }

                return;
            }
            // TODO 执行结束修改context的state,并且通知父类
            taskRunContext.changeContextState(ContextStateEnum.HAPPY_ENDING);
        }
    }
    public void append(BaseRunContext context, String content){
        if (context instanceof TaskRunContext taskRunContext) {
            content = content+ CommandConstant.ENTER;
            loggerManager.append(taskRunContext.getContextDef().getId(),content);
        }
    }
    public abstract void execute(TaskRunContext context);
    public void toBadEnding(){
        throw new ServiceException(GlobalErrorCodeConstants.PIPELINE_ERROR.getCode(),"");
    }
}
